/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX/HB9DRD
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	FSK441App_H
#define	FSK441App_H

#include <wx/wx.h>

#include "FSK441Frame.h"

#include "common/Text.h"

#include "fsk441/FSK441Controller.h"
#include "fsk441/FSK441Levels.h"
#include "fsk441/FSK441Spectrum.h"
#include "fsk441/FSK441Message.h"
#include "fsk441/FSK441FileReceive.h"

class CFSK441App : public wxApp {

    public:
	CFSK441App();
	virtual ~CFSK441App();

	virtual bool OnInit();
	virtual int  OnExit();
	
	// This is overridden because dialog boxes from threads are bad news
#ifdef __WXDEBUG__
	virtual void OnAssert(const wxChar* file, int line, const wxChar* cond, const wxChar* msg);
#endif
	virtual void receiveCardMessage(CFSK441Message* message) const;
	virtual void receiveFileMessage(CFSK441Message* message) const;

	virtual void showCardLevels(CFSK441Levels* levels) const;
	virtual void showFileLevels(CFSK441Levels* levels) const;

	virtual void showCardSpectrum(CFSK441Spectrum* spectrum) const;
	virtual void showFileSpectrum(CFSK441Spectrum* spectrum) const;

	virtual void error(const wxString& text) const;

	virtual bool     getTXFirst() const;
	virtual void     setTXFirst(bool first) const;
	virtual bool     getTXEnable() const;
	virtual void     setTXEnable(bool enabled);
	
	virtual int      getMinLength() const;
	virtual void     setMinLength(int length) const;
	virtual int      getMinStrength() const;
	virtual void     setMinStrength(int strength) const;
	virtual bool     getSingleTone() const;
	virtual void     setSingleTone(bool singleTone) const;

	virtual void     getPersonal(wxString& callsign, wxString& locator) const;
	virtual void     setPersonal(const wxString& callsign, const wxString& locator) const;
	virtual void     getPTT(wxString& type, wxString& device) const;
	virtual void     setPTT(const wxString& type, const wxString& device) const;
	virtual void     getSound(wxString& type, wxString& fileName, int& bits) const;
	virtual void     setSound(const wxString& type, const wxString& fileName, int bits) const;
	virtual wxString getReplacedMessage(int n) const;
	virtual wxString getOriginalMessage(int n) const;
	virtual void     setMessage(int n, const wxString& message);
	virtual void     getMessage(int n, wxString& message) const;

	virtual void     setRemote(const wxString& callsign, const wxString& locator);
	virtual void     setRecording(bool recording);
	virtual void     sendMessage(const wxString& message);
	virtual void     readFile(const wxString& fileName);

	virtual void     setSaving(bool saving);
	virtual void     logMessage(const CFSK441Message& message) const;
	virtual void     deleteMessages() const;

    private:
	CFSK441Frame*       m_frame;
	CFSK441Controller*  m_controller;
	CText**             m_messages;
	CFSK441FileReceive* m_file;
	bool                m_saving;
	bool                m_recording;
	bool                m_txEnable;

	void createController();
	void createFileReader();
	void readRegistry();
};

DECLARE_APP(CFSK441App)

#endif
