/*
 *   Copyright (C) 2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "NoisyToneGenerator.h"

#include "common/Exception.h"
#include "common/SoundFile.h"

#include "NoisyNCO.h"

int main(int argc, char *argv[])
{
	if (argc < 5) {
		::fprintf(stderr, "Usage: NoisyToneGenerator <frequency> <amplitude> <std dev> <length> <outfile>\n");
		return 1;
	}

	double       frequency  = ::atof(argv[1]);
	double       amplitude  = ::atof(argv[2]);
	double           stdDev = ::atof(argv[3]);
	unsigned int length     = ::atoi(argv[4]);
	wxString     fileName   = wxString(argv[5]);

	try {
		CNoisyToneGenerator noise(frequency, amplitude, stdDev, length, fileName);
		noise.run();
	}
	catch (CException& ex) {
		::fprintf(stderr, "NoisyToneGenerator: error: %s\n", ex.getMessage().c_str());
		return 1;
	}
	catch (...) {
		::fprintf(stderr, "NoisyToneGenerator: an error has occured\n");
		return 1;
	}

	return 0;
}

CNoisyToneGenerator::CNoisyToneGenerator(double frequency, double amplitude, double stdDev, unsigned int length, const wxString& fileName) :
m_frequency(frequency),
m_amplitude(amplitude),
m_stdDev(stdDev),
m_length(length),
m_fileName(fileName)
{
}

CNoisyToneGenerator::~CNoisyToneGenerator()
{
}

void CNoisyToneGenerator::run()
{
	CSoundFile out;
	out.openWrite(m_fileName, 11025, 8);

	int nSamples = 11025 * m_length;

	CNoisyNCO nco(11025, m_stdDev);

	double* tones = new double[nSamples];

	nco.generate(m_frequency, tones, nSamples);

	out.write(tones, nSamples, m_amplitude);

	out.close();

	delete[] tones;
}
